/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.rjconsultores.ventaboletos.web.gui.controladores.catalogos;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.utilerias.ApplicationProperties;
import com.rjconsultores.ventaboletos.utilerias.CustomEnum;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderEmpresa;
import java.util.HashMap;
import java.util.Map;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Comboitem;
import org.zkoss.zul.Listheader;
import org.zkoss.zul.Paging;
import org.zkoss.zul.Row;
import org.zkoss.zul.Textbox;

/**
 * 
 * @author Rafius
 */
@Controller("busquedaEmpresaController")
@Scope("prototype")
public class BusquedaEmpresaController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;
	@Autowired
	private transient PagedListWrapper<Empresa> plwEmpresa;
	private MyListbox empresaList;
	private Paging pagingEmpresa;
	private Textbox txtNombre;
	private Textbox txtEquivalencia;
	private Textbox txtRFC;
	private Combobox cmbIndTipo;
	private Row rowEquivalencia;
	private Listheader headerEquivalencia;

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);

		empresaList.setItemRenderer(new RenderEmpresa());
		empresaList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				Empresa emp = (Empresa) empresaList.getSelected();
				verPeriodo(emp);
			}
		});

		refreshLista();

		if (!ApplicationProperties.getInstance().isCustomHabilitado(CustomEnum.MOSTRAR_EQUIVALENCIA.getDescricao())) {
			rowEquivalencia.setVisible(Boolean.FALSE);
			headerEquivalencia.setVisible(Boolean.FALSE);
		}

		txtNombre.focus();
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void verPeriodo(Empresa emp) {
		if (emp == null) {
			return;
		}

		Map args = new HashMap();
		args.put("empresa", emp);
		args.put("empresaList", empresaList);

		openWindow("/gui/catalogos/editarEmpresa.zul",
				Labels.getLabel("editarEmpresaController.window.title"), args, MODAL);
	}

	private void refreshLista() {
		HibernateSearchObject<Empresa> empresaBusqueda =
				new HibernateSearchObject<Empresa>(Empresa.class, pagingEmpresa.getPageSize());

		Comboitem cbiTipo = cmbIndTipo.getSelectedItem();
		Short tipo = null;
		if (cbiTipo != null) {
			tipo = Short.parseShort(cbiTipo.getValue().toString());
			empresaBusqueda.addFilterEqual("indTipo", tipo);
		}

		if (!txtRFC.getText().isEmpty()) {
			empresaBusqueda.addFilterEqual("rfc", txtRFC.getText());
		}

		empresaBusqueda.addFilterNotEqual("empresaId", -1);
		empresaBusqueda.addFilterLike("nombempresa", "%" + txtNombre.getText().trim().concat("%"));

		if (!txtEquivalencia.getText().isEmpty()) {
			empresaBusqueda.addFilterEqual("equivalenciaId", txtEquivalencia.getText());
		}

		empresaBusqueda.addFilterEqual(ACTIVO, Boolean.TRUE);
		empresaBusqueda.addSortAsc("nombempresa");

		plwEmpresa.init(empresaBusqueda, empresaList, pagingEmpresa);

		if (empresaList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("busquedaEmpresaController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}

	public void onClick$btnPesquisa(Event ev) throws InterruptedException {
		refreshLista();
	}

	public void onClick$btnRefresh(Event ev) {
		refreshLista();
	}

	public void onClick$btnNovo(Event ev) {
		verPeriodo(new Empresa());
	}
}
