package com.rjconsultores.ventaboletos.web.gui.controladores.catalogos;

import java.util.HashMap;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Comboitem;
import org.zkoss.zul.Messagebox;
import org.zkoss.zul.Paging;
import org.zkoss.zul.Textbox;

import com.rjconsultores.ventaboletos.entidad.Ciudad;
import com.rjconsultores.ventaboletos.entidad.Escola;
import com.rjconsultores.ventaboletos.service.CiudadService;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderEscola;

@Controller("busquedaEscolaController")
@Scope("prototype")
public class BusquedaEscolaController  extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;
	@Autowired
	private CiudadService ciudadService;
	@Autowired
	private transient PagedListWrapper<Escola> plwEscola;
	private MyListbox escolaList;
	private Paging pagingEscola;
	private Textbox txtNombre;
	private Combobox cmbCiudad;


	@Override
	public void doAfterCompose(Component comp) throws Exception {

		super.doAfterCompose(comp);

		escolaList.setItemRenderer(new RenderEscola());
		escolaList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				Escola esc = (Escola) escolaList.getSelected();
				verEscola(esc);
			}
		});

		refreshLista();

		txtNombre.focus();
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void verEscola(Escola e) {
		if (e == null) {
			return;
		}

		Map args = new HashMap();
		args.put("escola", e);
		args.put("escolaList", escolaList);

		openWindow("/gui/catalogos/editarEscola.zul",
				Labels.getLabel("editarEscolaController.window.title"), args, MODAL);
	}

	private void refreshLista() throws InterruptedException {
		HibernateSearchObject<Escola> escolaBusqueda =
				new HibernateSearchObject<Escola>(Escola.class, pagingEscola.getPageSize());

		escolaBusqueda.addFilterLike("nombescola", "%" + txtNombre.getText().trim().concat("%"));

		Comboitem itemCiudad = cmbCiudad.getSelectedItem();
		if (itemCiudad != null) {
			Ciudad ciudad = (Ciudad) itemCiudad.getValue();
			escolaBusqueda.addFilterEqual("ciudad", ciudad);
		}

		escolaBusqueda.addFilterEqual(ACTIVO, Boolean.TRUE);
		escolaBusqueda.addSortAsc("nombescola");
		escolaBusqueda.addFilterNotEqual("escolaId", -1);

		plwEscola.init(escolaBusqueda, escolaList, pagingEscola);

		if (escolaList.getData().length == 0) {
			Messagebox.show(
					Labels.getLabel("MSG.ningunRegistro"),
					Labels.getLabel("busquedaEscolaController.window.title"),
					Messagebox.OK, Messagebox.INFORMATION);
		}
	}

	public void onClick$btnRefresh(Event ev) throws InterruptedException {
		refreshLista();
	}

	public void onClick$btnPesquisa(Event ev) throws InterruptedException {
		refreshLista();
	}

	public void onClick$btnNovo(Event ev) throws InterruptedException {
		verEscola(new Escola());
	}
}
