package com.rjconsultores.ventaboletos.web.gui.controladores.comissao;

import java.util.HashMap;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Paging;
import org.zkoss.zul.Textbox;

import com.rjconsultores.ventaboletos.entidad.TipoInformativoComissao;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderTipoInformativoComissao;

@Scope("prototype")
@Controller("busquedaTipoInformativoComissaoController")
public class BusquedaTipoInformativoComissaoController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;
	@Autowired
	private transient PagedListWrapper<TipoInformativoComissao> plwTipoInformativo;
	private MyListbox tipoInformativoList;
	private Paging pagingTipoInformativo;
	private Textbox txtNombre;

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);

		tipoInformativoList.setItemRenderer(new RenderTipoInformativoComissao());
		tipoInformativoList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				TipoInformativoComissao tc = (TipoInformativoComissao) tipoInformativoList.getSelected();
				verTipoInformativo(tc);
			}
		});

		refreshLista();

		txtNombre.focus();
	}

	public Paging getPagingTipoInformativo() {
		return pagingTipoInformativo;
	}

	public void setPagingTipoInformativo(Paging pagingTipoInformativo) {
		this.pagingTipoInformativo = pagingTipoInformativo;
	}

	public PagedListWrapper<TipoInformativoComissao> getPlwTipoInformativo() {
		return plwTipoInformativo;
	}

	public void setPlwTipoInformativo(PagedListWrapper<TipoInformativoComissao> plwTipoInformativo) {
		this.plwTipoInformativo = plwTipoInformativo;
	}

	public MyListbox getTipoInformativoList() {
		return tipoInformativoList;
	}

	public void setTipoInformativoList(MyListbox tipoInformativoList) {
		this.tipoInformativoList = tipoInformativoList;
	}

	public Textbox getTxtNombre() {
		return txtNombre;
	}

	public void setTxtNombre(Textbox txtNombre) {
		this.txtNombre = txtNombre;
	}

	public void onClick$btnPesquisa(Event ev) throws InterruptedException {
		refreshLista();
	}

	public void onClick$btnRefresh(Event ev) {
		refreshLista();
	}

	public void onClick$btnNovo(Event ev) {
		verTipoInformativo(new TipoInformativoComissao());
	}

	private void verTipoInformativo(TipoInformativoComissao tipoInformativoComissao) {
		if (tipoInformativoComissao == null) {
			return;
		}

		Map<String, Object> args = new HashMap<String, Object>();
		args.put("tipoInformativoComissao", tipoInformativoComissao);
		args.put("tipoInformativoComissaoList", tipoInformativoList);

		openWindow("/gui/catalogos/editarTipoInformativoComissao.zul",
				Labels.getLabel("busquedaTipoServicioController.window.title"), args, MODAL);
	}

	private void refreshLista() {
		HibernateSearchObject<TipoInformativoComissao> claseServicioBusqueda = new HibernateSearchObject<TipoInformativoComissao>(TipoInformativoComissao.class, pagingTipoInformativo.getPageSize());

		claseServicioBusqueda.addFilterLike("desctipo", "%" + txtNombre.getText().trim().concat("%"));
		claseServicioBusqueda.addFilterEqual(ACTIVO, Boolean.TRUE);
		claseServicioBusqueda.addSortAsc("desctipo");

		plwTipoInformativo.init(claseServicioBusqueda, tipoInformativoList, pagingTipoInformativo);
	}

}
