package com.rjconsultores.ventaboletos.web.gui.controladores.expressos;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zul.Checkbox;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Intbox;
import org.zkoss.zul.Paging;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.SolicitudExpreso;
import com.rjconsultores.ventaboletos.entidad.StatusSolicitudExpreso;
import com.rjconsultores.ventaboletos.service.StatusSolicitudExpresosService;
import com.rjconsultores.ventaboletos.web.utilerias.MyDatebox;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderExpresosPorCotizar;

@Controller("expressosPorCotizarController")
@Scope("prototype")
public class ExpressosPorCotizarController extends MyGenericForwardComposer{
	
	private static final long serialVersionUID = 1L;
	
	@Autowired
    private transient PagedListWrapper<SolicitudExpreso> plwTrayectosExpresos;
	
	@Autowired
	StatusSolicitudExpresosService statusSolicitudExpresosService;
	
	private List<Empresa> lsEmpresa;
	private Combobox cmbEmpresa;
	private MyListbox expresosList;
	private Paging pagingExpresos;
	private Checkbox ckServiciosInactivos;
	
	private MyDatebox txtFechaInicio;
	private MyDatebox txtFechaFin;
	
	private Intbox txtNumSolicitud;
	
	@Override
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);
		
		expresosList.setItemRenderer(new RenderExpresosPorCotizar(this));
		
		refreshLista();
	}
	
	private void refreshLista() {
		HibernateSearchObject<SolicitudExpreso> buscarExpresos =
                new HibernateSearchObject<SolicitudExpreso>(SolicitudExpreso.class, pagingExpresos.getPageSize());
		
		Date fechaInicio = txtFechaInicio.getValue();
		if(fechaInicio != null) {
			buscarExpresos.addFilterGreaterOrEqual("fechaSolicitud", fechaInicio);
		}
		
		Date fechaFin = txtFechaFin.getValue();
		if(fechaFin != null) {
			buscarExpresos.addFilterLessOrEqual("fechaSolicitud", fechaFin);
		}
		
		if(ckServiciosInactivos.isChecked()) {
			buscarExpresos.addFilterEqual(ACTIVO, true);
		}
		
		if(txtNumSolicitud.getValue() != null && txtNumSolicitud.getValue().toString() != "") {
			buscarExpresos.addFilterEqual("solicitudExpresoId", txtNumSolicitud.getValue());
		}
		
		List<Integer> filterStatusSolicitudExpreso = new ArrayList<Integer>();
		
		for(StatusSolicitudExpreso status : statusSolicitudExpresosService.obtenerTodos()) {
			switch (status.getDescStatusSolicitudExpreso()) {
				case "SOLICITADA": filterStatusSolicitudExpreso.add(status.getStatusSolicitudExpresoId()); break;
				case "COTIZACION_ENVIADA": filterStatusSolicitudExpreso.add(status.getStatusSolicitudExpresoId()); break;
			}
		}
		
		buscarExpresos.addFilterIn("statusSolicitudExpresoId", filterStatusSolicitudExpreso);
		
		plwTrayectosExpresos.init(buscarExpresos, expresosList, pagingExpresos);
	}
	
	public void onClick$btnRefresh(Event ev) {
        refreshLista();
    }
	
	public void onClick$btnPesquisa(Event ev) {
        refreshLista();
    }
	
	public List<Empresa> getLsEmpresa() {
		return lsEmpresa;
	}

	public void setLsEmpresa(List<Empresa> lsEmpresa) {
		this.lsEmpresa = lsEmpresa;
	}

	public Combobox getCmbEmpresa() {
		return cmbEmpresa;
	}

	public void setCmbEmpresa(Combobox cmbEmpresa) {
		this.cmbEmpresa = cmbEmpresa;
	}

	public MyListbox getExpresosList() {
		return expresosList;
	}

	public void setExpresosList(MyListbox expresosList) {
		this.expresosList = expresosList;
	}

	public Paging getPagingExpresos() {
		return pagingExpresos;
	}

	public void setPagingExpresos(Paging pagingExpresos) {
		this.pagingExpresos = pagingExpresos;
	}

	public MyDatebox getTxtFechaInicio() {
		return txtFechaInicio;
	}

	public void setTxtFechaInicio(MyDatebox txtFechaInicio) {
		this.txtFechaInicio = txtFechaInicio;
	}

	public PagedListWrapper<SolicitudExpreso> getPlwTrayectosExpresos() {
		return plwTrayectosExpresos;
	}

	public void setPlwTrayectosExpresos(PagedListWrapper<SolicitudExpreso> plwTrayectosExpresos) {
		this.plwTrayectosExpresos = plwTrayectosExpresos;
	}
}
