package com.rjconsultores.ventaboletos.web.gui.controladores.impressaofiscal;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Comboitem;
import org.zkoss.zul.Paging;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.FiscalAliquotaEmpresa;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;

@Controller("busquedaAliquotaECFController")
@Scope("prototype")
public class BusquedaAliquotaECFController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;
	@Autowired
	private transient PagedListWrapper<FiscalAliquotaEmpresa> plwFiscalAliquotaEmpresa;
	private Paging pagingFiscalAliquotaEmpresa;

	private Combobox cmbEmpresa;
	private List<Empresa> lsEmpresas;

	private MyListbox fiscalAliquotaEmpresaList;

	@Override
	public void doAfterCompose(Component comp) throws Exception {

		lsEmpresas = UsuarioLogado.getUsuarioLogado().getEmpresa();

		super.doAfterCompose(comp);

		fiscalAliquotaEmpresaList.setItemRenderer(new AliquotaECFListItemRenderer());
		fiscalAliquotaEmpresaList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				FiscalAliquotaEmpresa fiscalAliquotaEmpresa = (FiscalAliquotaEmpresa) fiscalAliquotaEmpresaList.getSelected();
				visualizaFiscalAliquotaEmpresa(fiscalAliquotaEmpresa);
			}
		});

		refreshLista();
	}

	private void visualizaFiscalAliquotaEmpresa(FiscalAliquotaEmpresa fiscalAliquotaEmpresa) {

		if (fiscalAliquotaEmpresa == null)
			return;

		Map<String, Object> args = new HashMap<String, Object>();
		args.put("fiscalAliquotaEmpresa", fiscalAliquotaEmpresa);
		args.put("fiscalAliquotaEmpresaList", fiscalAliquotaEmpresaList);
		args.put("busquedaAliquotaECFController", this);

		openWindow("/gui/impressaofiscal/editarAliquotaECF.zul",
				Labels.getLabel("editarAliquotaECFController.window.title"), args, MODAL);

	}

	public void onClick$btnPesquisa(Event ev) throws InterruptedException {
		refreshLista();
	}

	public void onClick$btnRefresh(Event ev) {
		refreshLista();
	}

	public void onClick$btnNovo(Event ev) {
		visualizaFiscalAliquotaEmpresa(new FiscalAliquotaEmpresa());
	}

	public void refreshLista() {
		HibernateSearchObject<FiscalAliquotaEmpresa> editarFormapagoEmpresaBusqueda =
				new HibernateSearchObject<FiscalAliquotaEmpresa>(FiscalAliquotaEmpresa.class, pagingFiscalAliquotaEmpresa.getPageSize());

		editarFormapagoEmpresaBusqueda.addFilterEqual(ACTIVO, true);

		Comboitem itemEmpresa = cmbEmpresa.getSelectedItem();
		if (itemEmpresa != null) {
			Empresa empresa = (Empresa) itemEmpresa.getValue();
			editarFormapagoEmpresaBusqueda.addFilterEqual("empresa", empresa);
		} else {
			editarFormapagoEmpresaBusqueda.addFilterIn("empresa", lsEmpresas);
		}

		plwFiscalAliquotaEmpresa.init(editarFormapagoEmpresaBusqueda, fiscalAliquotaEmpresaList, pagingFiscalAliquotaEmpresa);

		if (fiscalAliquotaEmpresaList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("busquedaAidfController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}

	public List<Empresa> getLsEmpresas() {
		return lsEmpresas;
	}

	public void setLsEmpresas(List<Empresa> _lsEmpresas) {
		lsEmpresas = _lsEmpresas;
	}

}
